/*
** ---------------------------------------------------------------
** drv.c - Module de controle do terceiro trabalho.
**
** ---------------------------------------------------------------
**
**    CIV 2801 - Fundamentos de Computacao Grafica Aplicada - 2006.2
**    Veja documentacao nas notas de aula.
**
** ---------------------------------------------------------------
**
** Autor: Luiz Fernando Martha
** Data: 08-Nov-2006
**
*/
#ifdef _WIN32
#include <windows.h>
#endif
#include <stdlib.h>
#include <math.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "iup.h"
#include "iupcontrols.h"
#include "iupgl.h"
#include "v3d.h"
#include "vec.h"

/*
** ---------------------------------------------------------------
** Variaveis:
*/
/* Objeto para o exemplo: um paralelepipedo com lados iguais
 * a 1, 2 e 3, e centrado em (0.5, 1.0, 1.5).
 */
static Point obj[8] =
         { { 0.0, 0.0, 0.0 },
           { 1.0, 0.0, 0.0 },
           { 1.0, 2.0, 0.0 },
           { 0.0, 2.0, 0.0 },
           { 0.0, 0.0, 3.0 },
           { 1.0, 0.0, 3.0 },
           { 1.0, 2.0, 3.0 },
           { 0.0, 2.0, 3.0 } };

/* Caixa que envolve o objeto.
 */
static double xmin = 0.0, xmax = 1.0;
static double ymin = 0.0, ymax = 2.0;
static double zmin = 0.0, zmax = 3.0;

/* Definicao de cores.
 */
static double color[][3] =
         { { 0.0, 1.0, 1.0 }, /* CYAN */
           { 0.0, 1.0, 0.0 }, /* GREEN */
           { 1.0, 1.0, 0.0 }  /* YELLOW */
         };

enum {
      COLOR_CYAN,
      COLOR_GREEN,
      COLOR_YELLOW
    };

/* Handle para o canvas.
 */
static Ihandle *canvas;

/* Handles para as listas de modelo, controle e tipo de visualizacao.
 */
static Ihandle *vismodellist;
static Ihandle *viscontrollist;
static Ihandle *vistypelist;

/* Modos de botao do mouse.
 */
#define ButtonPressed  1
#define ButtonReleased 0

/* Modo corrente do botao do mouse.
 */
static int mousebt_mode = ButtonReleased;

/*
** ---------------------------------------------------------------
** Funcoes locais e de callback:
*/
static void drvDisplaySolid( void );
static int  drvExit( Ihandle *self );
static void drvUpdateUILists( void );
static int  drvSelVisModel( Ihandle *self, char *t, int op, int v );
static int  drvSelVisControl( Ihandle *self, char *t, int op, int v );
static int  drvSelVisType( Ihandle *self, char *t, int op, int v );
static int  drvResize( Ihandle *self, int w, int h );
static int  drvRedraw( Ihandle *self );
static int  drvMouseBt( Ihandle *self, int b, int mode,
                        int x, int y, char *mousecfg );
static int  drvMouseMv( Ihandle *self, int x, int y, char *mousecfg );
static int  drvKeyCrtl( Ihandle *self, int key );
static int  drvIncrX( Ihandle *dial, double fac );
static int  drvIncrY( Ihandle *dial, double fac );
static int  drvIncrZ( Ihandle *dial, double fac );
static int  drvReset( Ihandle *self );
static int  drvFit( Ihandle *self );
static int  drvZoomIn( Ihandle *self );
static int  drvZoomOut( Ihandle *self );
static int  drvSave( Ihandle *self );
static int  drvRestore( Ihandle *self );

/* =======================  drvDisplaySolid  ===================== */

static void drvDisplaySolid( void )
{
 glBegin( GL_QUADS );
 /* Face -x */
 glColor3dv( color[COLOR_GREEN] );
 glNormal3d( -1.0, 0.0, 0.0 );
 glVertex3d( obj[0].x, obj[0].y, obj[0].z );
 glVertex3d( obj[4].x, obj[4].y, obj[4].z );
 glVertex3d( obj[7].x, obj[7].y, obj[7].z );
 glVertex3d( obj[3].x, obj[3].y, obj[3].z );
 
 /* Face -y */
 glColor3dv( color[COLOR_CYAN] );
 glNormal3d( 0.0, -1.0, 0.0 );
 glVertex3d( obj[0].x, obj[0].y, obj[0].z );
 glVertex3d( obj[1].x, obj[1].y, obj[1].z );
 glVertex3d( obj[5].x, obj[5].y, obj[5].z );
 glVertex3d( obj[4].x, obj[4].y, obj[4].z );
 
 /* Face -z */
 glColor3dv( color[COLOR_YELLOW] );
 glNormal3d( 0.0, 0.0, -1.0 );
 glVertex3d( obj[3].x, obj[3].y, obj[3].z );
 glVertex3d( obj[2].x, obj[2].y, obj[2].z );
 glVertex3d( obj[1].x, obj[1].y, obj[1].z );
 glVertex3d( obj[0].x, obj[0].y, obj[0].z );
 
 /* Face +x */
 glColor3dv( color[COLOR_GREEN] );
 glNormal3d( 1.0, 0.0, 0.0 );
 glVertex3d( obj[1].x, obj[1].y, obj[1].z );
 glVertex3d( obj[2].x, obj[2].y, obj[2].z );
 glVertex3d( obj[6].x, obj[6].y, obj[6].z );
 glVertex3d( obj[5].x, obj[5].y, obj[5].z );

 /* Face +y */
 glColor3dv( color[COLOR_CYAN] );
 glNormal3d( 0.0, 1.0, 0.0 );
 glVertex3d( obj[2].x, obj[2].y, obj[2].z );
 glVertex3d( obj[3].x, obj[3].y, obj[3].z );
 glVertex3d( obj[7].x, obj[7].y, obj[7].z );
 glVertex3d( obj[6].x, obj[6].y, obj[6].z );
 
 /* Face +z */
 glColor3dv( color[COLOR_YELLOW] );
 glNormal3d( 0.0, 0.0, 1.0 );
 glVertex3d( obj[4].x, obj[4].y, obj[4].z );
 glVertex3d( obj[5].x, obj[5].y, obj[5].z );
 glVertex3d( obj[6].x, obj[6].y, obj[6].z );
 glVertex3d( obj[7].x, obj[7].y, obj[7].z );
 glEnd( );
}

/* =========================  drvExit  ========================= */

static int drvExit( Ihandle *self )
{
 return( IUP_CLOSE );
}

/* =========================  drvUpdateUILists  ========================= */

static void drvUpdateUILists( void )
{
 char model;
 char control;
 char type;

 v3dGetModel( canvas, &model );
 switch( model )
 {
  case 's':
   IupSetAttribute( vismodellist, IUP_VALUE, "1" );
   break;
  case 'w':
   IupSetAttribute( vismodellist, IUP_VALUE, "2" );
   break;
  case 'o':
   IupSetAttribute( vismodellist, IUP_VALUE, "3" );
   break;
 }

 v3dGetControl( canvas, &control );
 switch( control )
 {
  case 'r':
   IupSetAttribute( viscontrollist, IUP_VALUE, "1" );
   break;
  case 't':
   IupSetAttribute( viscontrollist, IUP_VALUE, "2" );
   break;
 }

 v3dGetType( canvas, &type );
 switch( type )
 {
  case 'p':
   IupSetAttribute( vistypelist, IUP_VALUE, "1" );
   break;
  case 'o':
   IupSetAttribute( vistypelist, IUP_VALUE, "2" );
   break;
 }
}

/* =========================  drvSelVisModel  ========================= */

static int drvSelVisModel( Ihandle *self, char *t, int op, int v )
{
 if( !v )
  return( IUP_DEFAULT );

 switch( op )
 {
  case 1:        /* Screen based visualization model */
   v3dSetModel( canvas, 's' );
   break;
  case 2:        /* Walk-through visualization model */
   v3dSetModel( canvas, 'w' );
   break;
  case 3:        /* Object based visualization model */
   v3dSetModel( canvas, 'o' );
   break;
 }

 return( IUP_DEFAULT );
}

/* =========================  drvSelVisControl  ========================= */

static int drvSelVisControl( Ihandle *self, char *t, int op, int v )
{
 if( !v )
  return( IUP_DEFAULT );

 switch( op )
 {
  case 1:        /* Rotation visualization control */
   v3dSetControl( canvas, 'r' );
   break;
  case 2:        /* Translation visualization control */
   v3dSetControl( canvas, 't' );
   break;
 }

 return( IUP_DEFAULT );
}

/* =========================  drvSelVisType  ========================= */

static int drvSelVisType( Ihandle *self, char *t, int op, int v )
{
 if( !v )
  return( IUP_DEFAULT );

 switch( op )
 {
  case 1:        /* Perspective projection type */
   v3dSetType( canvas, 'p' );
   drvRedraw( canvas );
   break;
  case 2:        /* Orthographic projection type */
   v3dSetType( canvas, 'o' );
   drvRedraw( canvas );
   break;
 }

 return( IUP_DEFAULT );
}

/* =========================  drvResize  ========================= */

static int drvResize( Ihandle *self, int w, int h )
{
 IupGLMakeCurrent( self );
 glViewport( 0, 0, w-1, h-1 );
 glEnable( GL_LIGHTING );
 glEnable( GL_LIGHT0 );
 glLightModeli( GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE );
 glColorMaterial( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE );
 glEnable( GL_COLOR_MATERIAL );
 glEnable( GL_CULL_FACE );
 glDisable( GL_DEPTH_TEST );
 glClearColor( 1.0, 1.0, 1.0, 1.0 );

 v3dResize( canvas, w, h );

 return( IUP_DEFAULT );
}

/* =========================  drvRedraw  ========================= */

static int drvRedraw( Ihandle *self )
{
/* Tipo de projecao (perspectiva ou ortografica).
 */
 char proj_type;

/* Parametros da camera
 */
 double eyex, eyey, eyez;
 double refx, refy, refz;
 double vupx, vupy, vupz;

/* Parametros do volume de visao
 */
 double left, right;
 double bottom, top;
 double front, back;

/* Define ponto de luz na direcao de visao (eixo z do
 * sistema da camera e no infinito).
 */
 float light_pos[] = { 0.0F, 0.0F, 1.0F, 0.0F };

/* Define o tipo de projeo (perpectiva ou ortografica) 
 * e o volume de visao.
 */
 v3dGetType ( canvas, &proj_type );
 v3dGetViewVolume( canvas, &left, &right, &bottom, &top, &front, &back );
 glMatrixMode( GL_PROJECTION );
 glLoadIdentity( );
 if( proj_type == 'p' )
  glFrustum( left, right, bottom, top, front, back );
 else
  glOrtho( left, right, bottom, top, front, back );

/* Atualiza a camera.
 * Posiciona luz default branca na camera (antes de
 * aplicar a transformacao de camera).
 */
 v3dGetCamera( canvas, 
               &eyex, &eyey, &eyez, &refx, &refy, &refz, &vupx, &vupy, &vupz );
 glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
 glMatrixMode( GL_MODELVIEW );
 glLoadIdentity( );
 glLightfv( GL_LIGHT0, GL_POSITION, light_pos );
 gluLookAt( eyex, eyey, eyez, refx, refy, refz, vupx, vupy, vupz );

/* Redesenha objeto.
 */
 drvDisplaySolid( );
 IupGLSwapBuffers( self );

 return( IUP_DEFAULT );
}

/* =========================  drvMouseBt  ========================= */

static int drvMouseBt( Ihandle *self, int b, int mode,
                       int x, int y, char *mousecfg )
{
 if( mode == ButtonPressed )
  mousebt_mode = ButtonPressed;
 else
  mousebt_mode = ButtonReleased;

 v3dMouseBt( canvas, b, mode, x, y, mousecfg );

 return( IUP_DEFAULT );
}

/* =========================  drvMouseMv  ========================= */

static int drvMouseMv( Ihandle *self, int x, int y, char *mousecfg )
{
 if( mousebt_mode == ButtonPressed )
 {
  v3dMouseMv( canvas, x, y, mousecfg );
  drvRedraw( canvas );
 }
 return( IUP_DEFAULT );
}

/* =========================  drvKeyCrtl  ========================= */

static int drvKeyCrtl( Ihandle *self, int key )
{
 if( IupGetFocus( ) == canvas )
 {
  v3dKeyCrtl( canvas, key );
  drvRedraw( canvas );
  drvUpdateUILists( );
  return( IUP_IGNORE );
 }
 return( IUP_DEFAULT );
}

/* =====================  drvIncrX  ==================== */

static int drvIncrX( Ihandle *dial, double fac )
{
 char dir;
 if( fac < 0.0 )
  dir = '-';
 else
  dir = '+';
 v3dIncrCrtl( canvas, 'x', dir );
 drvRedraw( canvas );
 return( IUP_DEFAULT );
}

/* =====================  drvIncrY  ==================== */

static int drvIncrY( Ihandle *dial, double fac )
{
 char dir;
 if( fac < 0.0 )
  dir = '-';
 else
  dir = '+';
 v3dIncrCrtl( canvas, 'y', dir );
 drvRedraw( canvas );
 return( IUP_DEFAULT );
}

/* =====================  drvIncrZ  ==================== */

static int drvIncrZ( Ihandle *dial, double fac )
{
 char dir;
 if( fac < 0.0 )
  dir = '-';
 else
  dir = '+';
 v3dIncrCrtl( canvas, 'z', dir );
 drvRedraw( canvas );
 return( IUP_DEFAULT );
}

/* =========================  drvReset  ========================= */

static int drvReset( Ihandle *self )
{
 v3dDefault( canvas );
 drvRedraw( canvas );
 return( IUP_DEFAULT );
}

/* =========================  drvFit  ========================= */

static int drvFit( Ihandle *self )
{
 v3dFit( canvas );
 drvRedraw( canvas );
 return( IUP_DEFAULT );
}

/* =========================  drvZoomIn  ========================= */

static int drvZoomIn( Ihandle *self )
{
 v3dZoomIn( canvas );
 drvRedraw( canvas );
 return( IUP_DEFAULT );
}

/* =========================  drvZoomOut  ========================= */

static int drvZoomOut( Ihandle *self )
{
 v3dZoomOut( canvas );
 drvRedraw( canvas );
 return( IUP_DEFAULT );
}

/* =========================  drvSave  ========================= */

static int drvSave( Ihandle *self )
{
 v3dSave( canvas );
 return( IUP_DEFAULT );
}

/* =========================  drvRestore  ========================= */

static int drvRestore( Ihandle *self )
{
 v3dRestore( canvas );
 drvRedraw( canvas );
 drvUpdateUILists( );
 return( IUP_DEFAULT );
}

/* ==========================  main  ========================== */

int main( )
{
 Ihandle *dialog;
 static char msg_alarm[512];
 char *msg_error;

 IupOpen( );
 IupControlsOpen( );
 IupGLCanvasOpen( );

 msg_error = IupLoad( "trab3.led" );
 if( msg_error )
 {
  sprintf( msg_alarm, "\n%s\n", msg_error );
  IupAlarm( "Erro no arquivo trab3.led!", msg_alarm, " OK ", NULL, NULL );
  IupClose( );
  exit( 1 );
 }

/* Obtem os handles dos objetos de interface que podem ser manipulados.
 */
 dialog = IupGetHandle( "main" );
 canvas = IupGetHandle( "datascreen" );
 vismodellist = IupGetHandle( "vismodellist" );
 viscontrollist = IupGetHandle( "viscontrollist" );
 vistypelist = IupGetHandle( "vistypelist" );

/* Registra funcoes de callback para todas as correspondentes acoes.
 */
 IupSetFunction( "drvExit", (Icallback)drvExit );
 IupSetFunction( "drvSelVisModel", (Icallback)drvSelVisModel );
 IupSetFunction( "drvSelVisControl", (Icallback)drvSelVisControl );
 IupSetFunction( "drvSelVisType", (Icallback)drvSelVisType );
 IupSetFunction( "drvRedraw", (Icallback)drvRedraw );
 IupSetFunction( "drvResize", (Icallback)drvResize );
 IupSetFunction( "drvMouseBt", (Icallback)drvMouseBt );
 IupSetFunction( "drvMouseMv", (Icallback)drvMouseMv );
 IupSetFunction( "drvKeyCrtl", (Icallback)drvKeyCrtl );
 IupSetFunction( "drvIncrX", (Icallback)drvIncrX );
 IupSetFunction( "drvIncrY", (Icallback)drvIncrY );
 IupSetFunction( "drvIncrZ", (Icallback)drvIncrZ );
 IupSetFunction( "drvReset", (Icallback)drvReset );
 IupSetFunction( "drvFit", (Icallback)drvFit );
 IupSetFunction( "drvZoomIn", (Icallback)drvZoomIn );
 IupSetFunction( "drvZoomOut", (Icallback)drvZoomOut );
 IupSetFunction( "drvSave", (Icallback)drvSave );
 IupSetFunction( "drvRestore", (Icallback)drvRestore );

/* Inicializa modulo de visualizacao.
 */
 v3dOpen( canvas );
 v3dSetModelBox( canvas, xmin, xmax, ymin, ymax, zmin, zmax );
 v3dDefault( canvas );

/* Exibe dialogo de interface com usuario.
 */
 IupShow( dialog );

/* Passa o controle do programa para o IUP (para ficar 
 * respondendo atraves das funcoes de callback).
 */
 IupMainLoop( );

 v3dClose( canvas );
 IupClose( );
 return( 0 );
}

