/*
** ---------------------------------------------------------------
** vec.c  -  Package for implementing usual 3D vector operations.
**
** See public function prototypes and type definitions in header
** file vec.h
**
** ---------------------------------------------------------------
**
** remarks:
** ---------------------------------------------------------------
**
** In the header file "vec.h" there is a type definition for
** vectors:
**
** typedef struct
** {
**  double x,y,z;
** } Point, Vector;
**
** typedef struct
** {
**  double x,y,z,w;
** } Pnt4d, Vec4d;
**
** ---------------------------------------------------------------
**
** Version:    1.00
**
** Created:    08-Nov-2006  Luiz Fernando Martha
*/

/*
** ---------------------------------------------------------------
** Global variables and symbols
*/
#include <math.h>
#include "vec.h"


/*
** ---------------------------------------------------------------
** Local variables and symbols:
*/


/*
** ---------------------------------------------------------------
** Private functions:
*/


/*
** ---------------------------------------------------------------
** Public entry points:
*/

/*=========================  vecUnit  ==========================*/

void vecUnit( Vector *a, Vector *b )
{
 double len = sqrt(a->x*a->x + a->y*a->y + a->z*a->z);

 if( len != 0.0 )
 {
  b->x = a->x / len;
  b->y = a->y / len;
  b->z = a->z / len;
 }
}

/*==========================  vecDot  ==========================*/

double vecDot( Vector *a, Vector *b )
{
 return( a->x*b->x + a->y*b->y + a->z*b->z );
}

/*=========================  vecCross  =========================*/

void vecCross( Vector *a, Vector *b, Vector *c )
{
 c->x = a->y * b->z - a->z * b->y;
 c->y = a->z * b->x - a->x * b->z;
 c->z = a->x * b->y - a->y * b->x;
}
