/*
** ---------------------------------------------------------------
** vec.h:  Definitions and prototypes of package for 
**         implementing usual 3D vector operations.
**
** ---------------------------------------------------------------
**
** Public functions:
** ---------------------------------------------------------------
**
** void vecUnit( Vector *a, Vector *b )
**
**    a     -  given vector to normalize                      (in)
**    b     -  unit normalized vector                        (out)
**
** This function normalizes a given vector to a unit vector and 
** returns the result in another vector.
**
** ---------------------------------------------------------------
**
** double vecDot( Vector *a, Vector *b )
**
**    a,b   -  given vectors                                  (in)
**
** This function returns the dot product of the two given vectors.
**
** ---------------------------------------------------------------
**
** void vecCross( Vector *a, Vector *b, Vector *c )
**
**    a,b   -  given vectors                                  (in)
**    c     -  cross product vector                          (out)
**
** This function returns in the third vector parameter the cross 
** product of the two first vector parameters.
**
** ---------------------------------------------------------------
**
** Version:    1.00
**
** Created:    08-Nov-2006  Luiz Fernando Martha
**
*/

#ifndef _VEC_H
#define _VEC_H 

typedef struct 
{
  double x, y, z;
} Point, Vector;

typedef struct
{
  double x, y, z, w;
} Pnt4d, Vec4d;

#ifdef __cplusplus
extern "C" {
#endif

void   vecUnit ( Vector *a, Vector *b );
double vecDot  ( Vector *a, Vector *b );
void   vecCross( Vector *a, Vector *b, Vector *c );

#ifdef __cplusplus
}
#endif


#endif
